/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.FormHeaderTestFactory;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.ServiceKontoTestFactory;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelleTestFactory;

class VorgangServiceTest {

	@Spy
	@InjectMocks
	private VorgangService service;
	@Mock
	private GrpcEingangMapper eingangMapper;
	@Mock
	private VorgangRemoteService remoteService;

	@DisplayName("Create vorgang")
	@Nested
	class TestCreateVorgang {

		private final FormData formData = FormDataTestFactory.create();
		private final FormData preservedFormData = FormDataTestFactory.create();

		@BeforeEach
		void mockEingangMapper() {
			doReturn(preservedFormData).when(service).preserveConsistency(formData);
		}

		@Test
		void shouldPreserverConsistency() {
			callCreateVorgang();

			verify(service).preserveConsistency(formData);
		}

		@Test
		void shouldCallRemoteService() {
			callCreateVorgang();

			verify(remoteService).createVorgang(preservedFormData, Optional.ofNullable(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID));
		}

		private void callCreateVorgang() {
			service.createVorgang(formData);
		}
	}

	@DisplayName("Preserve consistency")
	@Nested
	class TestPreserveConsistency {

		private final ServiceKonto serviceKonto = ServiceKontoTestFactory.createBuilder().clearPostfachAddresses().build();
		private final FormHeader header = FormHeaderTestFactory.createBuilder().serviceKonto(serviceKonto).build();
		private final FormData formData = FormDataTestFactory.createBuilder().header(header).build();

		@DisplayName("should set serviceKonto to null if no postfachAddress exists")
		@Test
		void shouldSetServiceKontoToNull() {
			var consistentFormData = service.preserveConsistency(formData);

			assertThat(consistentFormData.getHeader().getServiceKonto()).isNull();
		}

		@DisplayName("should keep serviceKonto if postfachAddress eixsts")
		@Test
		void shouldKeepServiceKonto() {
			var consistentFormData = service.preserveConsistency(FormDataTestFactory.create());

			assertThat(consistentFormData.getHeader().getServiceKonto()).isEqualTo(FormHeaderTestFactory.SERVICE_KONTO);
		}

		@DisplayName("should proceed without exception on missing serviceKonto")
		@Test
		void shouldProceedWithoutException() {
			var formDataWithoutServiceKonto = FormDataTestFactory.createBuilder()
					.header(FormHeaderTestFactory.createBuilder().serviceKonto(null).build()).build();

			assertDoesNotThrow(() -> service.preserveConsistency(formDataWithoutServiceKonto));
		}
	}
}