package de.ozgcloud.eingang.semantik.enginebased.afm;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.AntragstellerTestFactory;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.semantik.common.ServiceKontoFactory;

class AfmAntragstellerHeaderMapperTest {

	@Spy
	@InjectMocks
	private AfmAntragstellerHeaderMapper mapper;

	@Nested
	class TestParseAntragstellerData {

		@Test
		void shouldCallGetHeaders() {
			var formData = FormData.builder().build();
			doReturn(Collections.emptyMap()).when(mapper).getHeaders(any());

			mapper.parseAntragstellerData(formData);

			verify(mapper).getHeaders(formData);
		}

		@Test
		void shouldCallBuildAntragsteller() {
			var headerMap = AfmHeaderTestFactory.createCustomHeaderMap();
			doReturn(headerMap).when(mapper).getHeaders(any());

			mapper.parseAntragstellerData(FormData.builder().build());

			verify(mapper).buildAntragsteller(headerMap);
		}

		@Test
		void shouldSetAntragsteller() {
			var antragsteller = AntragstellerTestFactory.create();
			doReturn(antragsteller).when(mapper).buildAntragsteller(any());

			var result = mapper.parseAntragstellerData(FormData.builder().build());

			assertThat(result.getAntragsteller()).isEqualTo(antragsteller);
		}

		@Nested
		class TestBuildAntragsteller {

			private Map<String, Object> headers = AfmHeaderTestFactory.createCustomHeaderMap();
			@Test
			void shouldSetPostfachId() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getPostfachId()).isEqualTo(AfmHeaderTestFactory.CUSTOM_POSTFACH_ID);
			}

			@Test
			void shouldSetVorname() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getVorname()).isEqualTo(AfmHeaderTestFactory.CUSTOM_VORNAME);
			}

			@Test
			void shouldSetNachname() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getNachname()).isEqualTo(AfmHeaderTestFactory.CUSTOM_NACHNAME);
			}

			@Test
			void shouldSetGeburtsname() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getGeburtsname()).isEqualTo(AfmHeaderTestFactory.CUSTOM_GEBURTSNAME);
			}

			@Test
			void shouldSetGeburtsort() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getGeburtsort()).isEqualTo(AfmHeaderTestFactory.CUSTOM_GEBURTSORT);
			}

			@Test
			void shoudlSetEmail() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getEmail()).isEqualTo(AfmHeaderTestFactory.CUSTOM_EMAIL);
			}

			@Test
			void shouldSetTelefon() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getTelefon()).isEqualTo(AfmHeaderTestFactory.CUSTOM_TELEFON);
			}

			@Test
			void shouldSetStrasse() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getStrasse()).isEqualTo(AfmHeaderTestFactory.CUSTOM_STRASSE);
			}

			@Test
			void shouldSetPlz() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getPlz()).isEqualTo(AfmHeaderTestFactory.CUSTOM_PLZ);
			}

			@Test
			void shouldSetOrt() {
				var result = mapper.buildAntragsteller(headers);

				assertThat(result.getOrt()).isEqualTo(AfmHeaderTestFactory.CUSTOM_ORT);
			}
		}
	}

	@Nested
	class TestIsResponsible {

		@Mock
		private FormData formData;

		@Test
		void shouldApproveResponsibility() {
			doReturn(true).when(mapper).isPostfachIdNotBlank(any());
			doReturn(createHeaders(ServiceKontoFactory.KEY_BAYERN_ID_POSTFACH_ID, "123")).when(mapper).getHeaders(any());

			var isResponsible = mapper.isResponsible(formData);

			assertTrue(isResponsible);
		}

		@Nested
		class TestDenyResponsibility {

			@Test
			void shouldDenyWhenNoHeader() {
				doReturn(Collections.emptyMap()).when(mapper).getHeaders(any());

				var isResponsible = mapper.isResponsible(FormData.builder().build());

				assertFalse(isResponsible);
			}

			@Test
			void shouldDenyWhenNoPostfachId() {
				doReturn(createHeaders(AfmAntragstellerHeaderMapper.KEY_VORNAME, "name")).when(mapper).getHeaders(any());

				var isResponsible = mapper.isResponsible(formData);

				assertFalse(isResponsible);
			}

			@Test
			void shouldDenyWhenPostfachIdIsBlank() {
				doReturn(createHeaders(ServiceKontoFactory.KEY_BAYERN_ID_POSTFACH_ID, null)).when(mapper).getHeaders(any());
				doReturn(false).when(mapper).isPostfachIdNotBlank(any());

				var isResponsible = mapper.isResponsible(formData);

				assertFalse(isResponsible);
			}

			@Nested
			class TestIsPostfachIdNotBlank {

				@ParameterizedTest
				@NullAndEmptySource
				void shouldReturnFalseWhenPostfachIdIsBlank(String postfachId) {
					var isNotBlank = mapper.isPostfachIdNotBlank(postfachId);

					assertFalse(isNotBlank);
				}

				@Test
				void shouldApprove() {
					var isNotBlank = mapper.isPostfachIdNotBlank("123");

					assertTrue(isNotBlank);
				}

				@Test
				void shouldFailOnUnexpectedType() {
					var postfachId = new Object();

					assertThrows(TechnicalException.class, () -> mapper.isPostfachIdNotBlank(postfachId));
				}
			}
		}
	}

	@Nested
	class TestGetHeaders {

		@Test
		void shouldReturnHeaders() {
			var headers = createHeaders(ServiceKontoFactory.KEY_BAYERN_ID_POSTFACH_ID, "123");
			var formData =  FormData.builder().formData(createHeaders(AfmHeaderMapper.HEADER_FIELD, headers)).build();

			var result = mapper.getHeaders(formData);

			assertThat(result).isEqualTo(headers);
		}

		@Test
		void shouldReturnEmptyMapWhenNoHeaders() {
			var formData = FormData.builder().build();

			var result = mapper.getHeaders(formData);

			assertThat(result).isEmpty();
		}
	}

	private Map<String, Object> createHeaders(String key, Object value) {
		var map = new HashMap<String, Object>();
		map.put(key, value);
		return map;
	}
}